// any or all of the route maps - March 2011
// ported to maplibre GL - August-November 2018

var status_helper = {main: false, daily: [], changed: false, maps: [], sent: [], lasttime: performance.now()};
var img_preload = [];

// extras may contain "forpdf" or be null
function Summary_Map(extras) {
    if(extras && extras.forpdf) {
	var target = $$('pdf_summary_map');
	local_processing.map_id = 0;
    } else {
	Any_Closeup_Maps();
	var target = $$('map');
	if(!target)
	    return;        // so harmless if no canvass
    }
    var conf = {target:target, map:"summary"};
    jQuery.extend(conf, extras);
    Async_JSON("../cgi-bin/api.cgi?mode=routemap&marker=no&session="+cp_info.session, Do_Map, conf);
}

function Daily_Maps(num, extras) {
    for(i=1; i<= num; i++) {
	Daily_Map(i, extras);
    }
}

function Daily_Map(num, extras) {
    if(extras && extras.forpdf) {
	var target = $$('pdf_daily_map');
	local_processing.map_id = num;
    } else {
	var target = $$('map_'+num);
	if(!target)
	    return;        // turned on and off by presence of divs
    }
    var conf = {target: target, map: num};
    jQuery.extend(conf, extras);
    Async_JSON("../cgi-bin/api.cgi?mode=dailymap&marker=no&daynum="+num+"&session="+cp_info.session, Do_Map, conf);
}

function Do_Map(data, config) {
    if(data.error == 'notable') {
        Canalplan_Alert("map-no-table");
        return;
    }
    var linegeodata = Coordinates_To_Geodata(data.line);
    var vp = Dimensions_From_Geodata(config.target, linegeodata);
    var mapOptions = {
        container: config.target,
        style: cp_info.map_stylepath+data.stylefile,
        center: vp.center,
        zoom: vp.zoom * 0.98,   // a small margin for markers etc
        attributionControl: true
    };
    var myopts = {poi:false};
    if(config.forpdf == "yes") {
        mapOptions.preserveDrawingBuffer = true;
	mapOptions.interactive = false;
        myopts.nocontrol = true;
    } else {
        myopts.resizable = true;
    }
    var map = Create_Map(mapOptions, myopts);

    if(config.forpdf == "yes") {
	local_processing.map = map;
    }
    map.on('load',function() {
        map.addLayer({
            id: "CPRoute",
            type: "line",
            source: {
                type: "geojson",
                data: linegeodata
            },
            layout: {
                "line-join": "round",
                "line-cap": "round"
            },
            paint: {
                "line-color": "#3C33FF",
                "line-width": 5
            }
            // add on top of last background waterway layer but behind places
        },'cp-waterway-lake-excluded');  
        Add_Markers_Start(map,data.stops);
    });
    if(config.forpdf == 'yes') {
        map.on('data',function(e) {
            if(e.isSourceLoaded && e.sourceId == "stopping-markers" && e.sourceDataType != "metadata") {
                // wait a mo for drawing to complete
                setTimeout(function() {
		    local_processing.mapflags.dataflag = true;
                },200);
            }
        });
        map.on("render", function() {
            if(map.loaded()) {
	        local_processing.mapflags.renderflag = true;
                local_processing.mapflags.renderwait = performance.now();
            }
        });
    }
}

// in an ideal world we'd use promises for this.  But IE.
function Add_Markers_Start(map, stops) {
    var imagestoget = stops.length;
    // get all the icons into the map - when last done, call next stage
    for(i=0;i<stops.length;i++) {
        // anonymous function to create scope so idx is different every call
        (function(idx) {
            map.loadImage(imgurl+stops[idx].icon+".png", function(error,image) {
                if(error) throw(error);
                map.addImage(stops[idx].icon, image);
                if(--imagestoget == 0) {
                    Add_Markers_Second_Step(map,stops);
                }
            });
        })(i);
    }
}

function Add_Markers_Second_Step(map, stops) {
    var stopdat = [];
    for(i=0;i<stops.length;i++) {
        stopdat.push({
            "type": "feature",
            "geometry": {
                "type": "Point",
                "coordinates": [stops[i].lng, stops[i].lat]
            },
            "properties" : {
                "icon": stops[i].icon,
                "refid": stops[i].idx
            }
        });
    }
    map.addLayer({
        "id": "stopping-markers",
        "type": "symbol",
        "source": {
            "type": "geojson",
            "data": {
                "type": "FeatureCollection",
                "features": stopdat
            }
        },
        "layout": {
            "icon-image": "{icon}",
            "icon-anchor": "bottom",
            "icon-size": 1.2
        }
    });
    // attach pop-ups to the new layer - close clone of code in maplibre_mapstuff
    var markerpopup = new maplibregl.Popup({
        className: 'placeinfo-popup',
        closeButton: false,
        closeOnClick: true
    });
    map.on('mouseenter','stopping-markers',function(e) {
        //        if(popuplive)
        //            return;
        //        popuplive = true;
        map.getCanvas().style.cursor = 'pointer';
        var coordinates = e.features[0].geometry.coordinates.slice();
        markerpopup.setLngLat(coordinates)
            .setHTML(Get_Info_Content(e.features[0].properties.refid))
            .addTo(map);
    });
    
    //    map.on('mouseleave', 'stopping-markers', function() {
    //        map.getCanvas().style.cursor = '';
    //        markerpopup.remove();
    //        popuplive = false;
    //    });
}  

function Get_Info_Content(idx) {
    var hd = steveqa('day_header_'+idx);
    var bd = steveqa('day_body_'+idx);
    var toret = '';
    if (typeof(hd.innerHTML)!= "undefined") 
	toret += hd.innerHTML;
      if (typeof(bd.innerHTML)!= "undefined") {
	if(toret)
	    toret += '<br>';
	toret += bd.innerHTML;
    }
    return toret;
}

function Send_Closeup_Map_Mainfest() {
    var manifest = [];
    var el = document.getElementsByClassName('closeup_map');
    for(let i=0; i<el.length; i++) {
	var plid = el[i].getAttribute('data-placeid');
	manifest.push(plid);
    }
    var fd = new FormData();
    fd.append("mode","closeup_manifest");
    fd.append("manifest", manifest);
    fd.append("filename","manifest");
    var xhr = new XMLHttpRequest();
    xhr.open('POST', '/cgi-bin/callback.cgi', true);
    xhr.send(fd);
    xhr.addEventListener("load", function() {
        //	      console.log("Successfully sent ",manifest);
        document.location = '/cgi-bin/canal.cgi?dopdf=yes';
    });
}

function Itinerary_Maps(n) {
    // each of these uses an asynchronous data fetch so all will run at once
    Summary_Map();
    Daily_Maps();
}

function Generate_Route_PDF() {
    Send_Closeup_Map_Mainfest();
}
